'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var self = microWorker;
module.exports = self;
var uuid = require('node-uuid');
var util = require('util');
var ApiAdapter = require('./Adapter.js');
var prepData = require('./_common/prepData.js');
var sendApprovalRequests = require('./_common/sendApprovalRequests.js');
var queueStepsToNodePools = require('./_common/queueStepsToNodePools.js');
var queueExternalSteps = require('./_common/queueExternalSteps.js');
var checkExecutionCapacity = require('./_common/checkExecutionCapacity.js');
var sendRejectedStepletConsoles = require('./_common/sendRejectedStepletConsoles.js');
const logger_1 = require("@jfrog-pipelines/logger");
const checkExecutionCapacityNew_js_1 = require("./_common/checkExecutionCapacityNew.js");
const queueExternalStepsNew_js_1 = require("./_common/queueExternalStepsNew.js");
const SendApprovalRequest_1 = require("./_common/SendApprovalRequest");
const { EXPERIMENTAL_PROPERTIES_CONFIG_NAME } = require('./_common/constants');
function microWorker(message, callback) {
    var bag = {
        message: message,
        jFrogPipelinesSessionId: message.payload &&
            message.payload.jFrogPipelinesSessionId || uuid.v4(),
        finalStatesMap: {
            'success': 'Successful',
            'failure': 'Failed',
            'error': 'Error',
            'skipped': 'Skipped',
            'timeout': 'TimedOut',
            'cancelled': 'Cancelled',
            'unstable': 'Unstable',
            'autosyncfailure': 'Failure-AutoSync'
        },
        externalStepTypes: ['Jenkins'],
        incompleteSteps: [],
        incompleteSteplets: [],
        waitingGroups: [],
        processingGroups: [],
        pipelines: [],
        runs: [],
        runStepConnections: [],
        availableNodePools: [],
        hasRemainingCapacity: false,
        defaultPriority: 9999,
        rejectedGroups: {
            waitingForPreviousRunToFinish: [],
            waitingForParentPipelineToComplete: [],
            waitingForParentPipelineToSuccess: [],
            waitingForChildPipelineToComplete: [],
            waitingForInputStepsToComplete: [],
            waitingForFreeNodePoolCapacity: [],
            waitingForFreeLicenseCapacity: [],
            waitingForApproval: []
        },
    };
    var startTime = Date.now();
    bag.who = util.format('nod|stepService|%s|jFrogPipelinesSessionId:%s', self.name, bag.jFrogPipelinesSessionId);
    logger.debug(bag.who, 'Starting');
    async.series([
        _checkInputParams.bind(null, bag),
        _getExperimentalProperties.bind(null, bag),
        _checkExecutionCapacity.bind(null, bag),
        _prepData.bind(null, bag),
        _sendApprovalRequests.bind(null, bag),
        _queueStepsToNodePools.bind(null, bag),
        _queueExternalSteps.bind(null, bag),
        _queueExternalStepsWithRunAdjacency.bind(null, bag),
        _sendRejectedStepletConsoles.bind(null, bag)
    ], function (err) {
        var stopTime = Date.now();
        var timeDiff = new Date(stopTime - startTime);
        var minutes = timeDiff.getMinutes();
        var seconds = timeDiff.getSeconds();
        var ms = timeDiff.getMilliseconds();
        var finalTime = util.format('%s minutes, %s seconds, and %s milliseconds', minutes, seconds, ms);
        let stepTriggerStatus = '';
        if (err) {
            stepTriggerStatus = 'failed';
            logger.warn(bag.who, 'Failed to process step trigger request -', finalTime);
        }
        else {
            stepTriggerStatus = 'success';
            logger.debug(bag.who, 'Successfully processed step trigger request -', finalTime);
        }
        const eventData = {
            timestamp: new Date(stopTime).toISOString(),
            domain: global.msName,
            eventType: 'steptriggeriteration',
            status: stepTriggerStatus,
            durationMillis: timeDiff.valueOf(),
            pipelinesSessionId: bag.jFrogPipelinesSessionId
        };
        logger_1.LoggerFactory.logEvent(eventData);
        return callback();
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.debug(who, 'Inside');
    bag.statusCodes = global.systemCodesByGroup.status;
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    return next();
}
function _getExperimentalProperties(bag, next) {
    var who = bag.who + '|' + _getExperimentalProperties.name;
    logger.debug(who, 'Inside');
    bag.suAdapter.getSystemConfigByName(EXPERIMENTAL_PROPERTIES_CONFIG_NAME, function (err, { configValue }) {
        if (err)
            return next(err);
        bag.stepTriggerLicenseCheckLockRemoved =
            configValue.stepTriggerLicenseCheckLockRemoved || false;
        logger.trace(who, 'current stepTriggerLicenseCheckLockRemoved: ', bag.stepTriggerLicenseCheckLockRemoved);
        bag.stepTriggerLicenseCheckHueristicsEnabled =
            configValue.stepTriggerLicenseCheckHueristicsEnabled || false;
        logger.trace(who, 'current stepTriggerLicenseCheckHueristicsEnabled: ', bag.stepTriggerLicenseCheckHueristicsEnabled);
        bag.useRunAdjacency = configValue.useRunAdjacency || false;
        return next();
    });
}
async function _checkExecutionCapacity(bag) {
    var who = bag.who + '|' + _checkExecutionCapacity.name;
    logger.debug(who, 'Inside');
    var innerBag = {
        who: who,
        suAdapter: bag.suAdapter,
        stepTriggerLicenseCheckHueristicsEnabled: bag.stepTriggerLicenseCheckHueristicsEnabled
    };
    const checkExecutionCapacity = new checkExecutionCapacityNew_js_1.CheckExecutionCapacity();
    const results = await checkExecutionCapacity.checkCapacity();
    bag.availableNodePools = results.availableNodePools;
    bag.hasRemainingCapacity = results.hasRemainingCapacity;
    bag.stepTriggerHueristics = {
        enabled: false,
    };
    bag.allNodePools = results.allNodePools;
    if (!bag.hasRemainingCapacity)
        logger.debug(who, 'No remaining capacity to process the steps');
    if (_.isEmpty(bag.availableNodePools))
        logger.debug(who, 'There are no free node pools that can process the steps');
}
function _prepData(bag, next) {
    var who = bag.who + '|' + _prepData.name;
    logger.debug(who, 'Inside');
    var innerBag = {
        who: who,
        suAdapter: bag.suAdapter,
        hasRemainingCapacity: bag.hasRemainingCapacity,
        availableNodePools: bag.availableNodePools,
        rejectedGroups: bag.rejectedGroups,
        allNodePools: bag.allNodePools,
        defaultPriority: bag.defaultPriority,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
        useRunAdjacency: bag.useRunAdjacency,
    };
    prepData(innerBag, function (err, result) {
        if (err) {
            logger.error(who, `prepData failed with error: ${util.inspect(err)}`);
            return next(err);
        }
        bag.incompleteStepsById = result.incompleteStepsById;
        bag.incompleteStepletsById = result.incompleteStepletsById;
        bag.incompleteStepletsByStepId = result.incompleteStepletsByStepId;
        bag.incompleteStepsByRunId = result.incompleteStepsByRunId;
        bag.waitingGroups = result.waitingGroups;
        bag.processingGroups = result.processingGroups;
        bag.pipelines = result.pipelines;
        bag.pipelinesById = result.pipelinesById;
        bag.runStepConnections = result.runStepConnections;
        bag.runStepConnectionsByStepId = result.runStepConnectionsByStepId;
        bag.runStepConnectionsByRunId = result.runStepConnectionsByRunId;
        bag.runResourceVersionsById = result.runResourceVersionsById;
        bag.runResourceVersionsByRunId = result.runResourceVersionsByRunId;
        bag.groupsProjects = result.groupsProjects;
        bag.allowedNodePoolsByProject = result.allowedNodePoolsByProject;
        bag.nodePools = result.nodePools;
        bag.orphanedOnDemandNodesByProjectId =
            result.orphanedOnDemandNodesByProjectId;
        bag.runsById = result.runsById;
        bag.rejectedGroups = result.rejectedGroups;
        return next();
    });
}
function _sendApprovalRequests(bag, next) {
    if (_.isEmpty(bag.waitingGroups))
        return next();
    var who = bag.who + '|' + _sendApprovalRequests.name;
    logger.debug(who, 'Inside');
    var _bag = {
        who: who,
        suAdapter: bag.suAdapter,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
        waitingGroups: bag.waitingGroups,
        rejectedGroups: bag.rejectedGroups,
        useRunAdjacency: bag.useRunAdjacency
    };
    if (!bag.useRunAdjacency) {
        return sendApprovalRequests(_bag, function (err, result) {
            if (err) {
                logger.error(who, `sendApprovalRequests failed with error: ${util.inspect(err)}`);
                return next();
            }
            bag.waitingGroups = result.waitingGroups;
            return next();
        });
    }
    const newSendApprovalRequests = new SendApprovalRequest_1.SendApprovalRequests(bag.jFrogPipelinesSessionId);
    newSendApprovalRequests.getWaitingAndRejectedGroupsAfterApprovalProcessing(bag.waitingGroups)
        .then((waitingAndRejectedGroups) => {
        bag.waitingGroups = waitingAndRejectedGroups.waitingGroups;
        bag.rejectedGroups.waitingForApproval = waitingAndRejectedGroups.rejectedGroupsForApproval;
        return next();
    })
        .catch((error) => {
        logger.error(who, util.inspect(error));
        return next();
    });
}
function _queueStepsToNodePools(bag, next) {
    if (_.isEmpty(bag.nodePools))
        return next();
    var who = bag.who + '|' + _queueStepsToNodePools.name;
    logger.debug(who, 'Inside');
    var _bag = {
        who: who,
        suAdapter: bag.suAdapter,
        statusCodes: global.systemCodesByGroup.status,
        nodePools: bag.nodePools,
        processingGroups: bag.processingGroups,
        waitingGroups: bag.waitingGroups,
        groupsProjects: bag.groupsProjects,
        allowedNodePoolsByProject: bag.allowedNodePoolsByProject,
        pipelinesById: bag.pipelinesById,
        incompleteStepsByRunId: bag.incompleteStepsByRunId,
        incompleteStepsById: bag.incompleteStepsById,
        runStepConnections: bag.runStepConnections,
        runStepConnectionsByRunId: bag.runStepConnectionsByRunId,
        runStepConnectionsByStepId: bag.runStepConnectionsByStepId,
        runResourceVersionsById: bag.runResourceVersionsById,
        runResourceVersionsByRunId: bag.runResourceVersionsByRunId,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
        orphanedOnDemandNodesByProjectId: bag.orphanedOnDemandNodesByProjectId,
        defaultPriority: bag.defaultPriority,
        runsById: bag.runsById,
        rejectedGroups: bag.rejectedGroups,
        stepTriggerLicenseCheckLockRemoved: bag.stepTriggerLicenseCheckLockRemoved,
        useRunAdjacency: bag.useRunAdjacency,
        stepTriggerHueristics: bag.stepTriggerHueristics
    };
    queueStepsToNodePools(_bag, function (err, result) {
        if (err) {
            logger.error(who, `queueStepsToNodePools failed with error: ${util.inspect(err)}`);
            return next(err);
        }
        bag.waitingGroups = result.waitingGroups;
        bag.processingGroups = result.processingGroups;
        return next();
    });
}
function _queueExternalSteps(bag, next) {
    if (_.isEmpty(bag.waitingGroups))
        return next();
    if (bag.useRunAdjacency)
        return next();
    var who = bag.who + '|' + _queueExternalSteps.name;
    logger.debug(who, 'Inside');
    var _bag = {
        who: who,
        suAdapter: bag.suAdapter,
        statusCodes: global.systemCodesByGroup.status,
        waitingGroups: bag.waitingGroups,
        groupsProjects: bag.groupsProjects,
        incompleteStepsById: bag.incompleteStepsById,
        runStepConnections: bag.runStepConnections,
        runStepConnectionsByRunId: bag.runStepConnectionsByRunId,
        runResourceVersionsByRunId: bag.runResourceVersionsByRunId,
        runStepConnectionsByStepId: bag.runStepConnectionsByStepId,
        runResourceVersionsById: bag.runResourceVersionsById,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
        defaultPriority: bag.defaultPriority,
        rejectedGroups: bag.rejectedGroups,
        stepTriggerLicenseCheckLockRemoved: bag.stepTriggerLicenseCheckLockRemoved,
        stepTriggerHueristics: bag.stepTriggerHueristics,
        useRunAdjacency: bag.useRunAdjacency
    };
    queueExternalSteps(_bag, function (err) {
        if (err) {
            logger.error(who, `queueExternalSteps failed with error: ${util.inspect(err)}`);
            return next(err);
        }
        return next();
    });
}
async function _queueExternalStepsWithRunAdjacency(bag) {
    if (_.isEmpty(bag.waitingGroups))
        return;
    if (!bag.useRunAdjacency)
        return;
    var who = bag.who + '|' + _queueExternalSteps.name;
    logger.debug(who, 'Inside');
    const externalStepWorker = new queueExternalStepsNew_js_1.QueueExternalSteps(bag.waitingGroups, bag.jFrogPipelinesSessionId, global.config.apiToken);
    try {
        await externalStepWorker.queueExternalSteps();
    }
    catch (error) {
        logger.warn(`Cannot queue external steps. Error`, error);
        throw error;
    }
}
function _sendRejectedStepletConsoles(bag, next) {
    var who = bag.who + '|' + _sendRejectedStepletConsoles.name;
    logger.debug(who, 'Inside');
    let _bag = {
        who: who,
        suAdapter: bag.suAdapter,
        rejectedGroups: bag.rejectedGroups,
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
    };
    sendRejectedStepletConsoles(_bag, function (err) {
        if (err) {
            logger.error(who, `sendRejectedStepletConsoles failed with error: ${util.inspect(err)}`);
            return next(err);
        }
        return next();
    });
}
//# sourceMappingURL=microWorker.js.map